"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class BankCardData(TLObject):
    """TL type: payments.bankCardData"""
    CONSTRUCTOR_ID = 0x3e24e573
    SUBCLASS_OF_ID = 0x8c6dd68b

    def __init__(self, title: str, open_urls: List['TypeBankCardOpenUrl']):
        self.title = title
        self.open_urls = open_urls

    def to_dict(self):
        return {
            '_': 'BankCardData',
            'title': self.title,
            'open_urls': self.open_urls,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.open_urls)))
        for item in self.open_urls:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        reader.read_int(signed=False)  # skip vector id
        _count_open_urls = reader.read_int()
        _list_open_urls = []
        for _ in range(_count_open_urls):
            _item_open_urls = reader.tgread_object()
            _list_open_urls.append(_item_open_urls)
        _args['open_urls'] = _list_open_urls
        return cls(**_args)


class CheckCanSendGiftResultFail(TLObject):
    """TL type: payments.checkCanSendGiftResultFail"""
    CONSTRUCTOR_ID = 0xd5e58274
    SUBCLASS_OF_ID = 0x632efa30

    def __init__(self, reason: 'TypeTextWithEntities'):
        self.reason = reason

    def to_dict(self):
        return {
            '_': 'CheckCanSendGiftResultFail',
            'reason': self.reason,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.reason))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_reason = reader.tgread_object()
        _args['reason'] = _val_reason
        return cls(**_args)


class CheckCanSendGiftResultOk(TLObject):
    """TL type: payments.checkCanSendGiftResultOk"""
    CONSTRUCTOR_ID = 0x374fa7ad
    SUBCLASS_OF_ID = 0x632efa30

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'CheckCanSendGiftResultOk',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class CheckedGiftCode(TLObject):
    """TL type: payments.checkedGiftCode"""
    CONSTRUCTOR_ID = 0xeb983f8f
    SUBCLASS_OF_ID = 0x5b2997e8

    def __init__(self, date: Optional[datetime], days: int, chats: List['TypeChat'], users: List['TypeUser'], via_giveaway: Optional[bool] = None, from_id: Optional['TypePeer'] = None, giveaway_msg_id: Optional[int] = None, to_id: Optional[int] = None, used_date: Optional[datetime] = None):
        self.date = date
        self.days = days
        self.chats = chats
        self.users = users
        self.via_giveaway = via_giveaway
        self.from_id = from_id
        self.giveaway_msg_id = giveaway_msg_id
        self.to_id = to_id
        self.used_date = used_date

    def to_dict(self):
        return {
            '_': 'CheckedGiftCode',
            'date': self.date,
            'days': self.days,
            'chats': self.chats,
            'users': self.users,
            'via_giveaway': self.via_giveaway,
            'from_id': self.from_id,
            'giveaway_msg_id': self.giveaway_msg_id,
            'to_id': self.to_id,
            'used_date': self.used_date,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.via_giveaway:
            flags |= (1 << 2)
        if self.from_id is not None:
            flags |= (1 << 4)
        if self.giveaway_msg_id is not None:
            flags |= (1 << 3)
        if self.to_id is not None:
            flags |= (1 << 0)
        if self.used_date is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.date))
        buf.write(struct.pack('<i', self.days))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.from_id is not None:
            buf.write(bytes(self.from_id))
        if self.giveaway_msg_id is not None:
            buf.write(struct.pack('<i', self.giveaway_msg_id))
        if self.to_id is not None:
            buf.write(struct.pack('<q', self.to_id))
        if self.used_date is not None:
            buf.write(struct.pack('<i', self.used_date))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['via_giveaway'] = bool(flags & (1 << 2))
        if flags & (1 << 4):
            _val_from_id = reader.tgread_object()
            _args['from_id'] = _val_from_id
        else:
            _args['from_id'] = None
        if flags & (1 << 3):
            _val_giveaway_msg_id = reader.read_int()
            _args['giveaway_msg_id'] = _val_giveaway_msg_id
        else:
            _args['giveaway_msg_id'] = None
        if flags & (1 << 0):
            _val_to_id = reader.read_long()
            _args['to_id'] = _val_to_id
        else:
            _args['to_id'] = None
        _val_date = reader.read_int()
        _args['date'] = _val_date
        _val_days = reader.read_int()
        _args['days'] = _val_days
        if flags & (1 << 1):
            _val_used_date = reader.read_int()
            _args['used_date'] = _val_used_date
        else:
            _args['used_date'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ConnectedStarRefBots(TLObject):
    """TL type: payments.connectedStarRefBots"""
    CONSTRUCTOR_ID = 0x98d5ea1d
    SUBCLASS_OF_ID = 0x235e1a67

    def __init__(self, count: int, connected_bots: List['TypeConnectedBotStarRef'], users: List['TypeUser']):
        self.count = count
        self.connected_bots = connected_bots
        self.users = users

    def to_dict(self):
        return {
            '_': 'ConnectedStarRefBots',
            'count': self.count,
            'connected_bots': self.connected_bots,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.connected_bots)))
        for item in self.connected_bots:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_connected_bots = reader.read_int()
        _list_connected_bots = []
        for _ in range(_count_connected_bots):
            _item_connected_bots = reader.tgread_object()
            _list_connected_bots.append(_item_connected_bots)
        _args['connected_bots'] = _list_connected_bots
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ExportedInvoice(TLObject):
    """TL type: payments.exportedInvoice"""
    CONSTRUCTOR_ID = 0xaed0cbd9
    SUBCLASS_OF_ID = 0x36105432

    def __init__(self, url: str):
        self.url = url

    def to_dict(self):
        return {
            '_': 'ExportedInvoice',
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class GiveawayInfo(TLObject):
    """TL type: payments.giveawayInfo"""
    CONSTRUCTOR_ID = 0x4367daa0
    SUBCLASS_OF_ID = 0x96a377bd

    def __init__(self, start_date: Optional[datetime], participating: Optional[bool] = None, preparing_results: Optional[bool] = None, joined_too_early_date: Optional[datetime] = None, admin_disallowed_chat_id: Optional[int] = None, disallowed_country: Optional[str] = None):
        self.start_date = start_date
        self.participating = participating
        self.preparing_results = preparing_results
        self.joined_too_early_date = joined_too_early_date
        self.admin_disallowed_chat_id = admin_disallowed_chat_id
        self.disallowed_country = disallowed_country

    def to_dict(self):
        return {
            '_': 'GiveawayInfo',
            'start_date': self.start_date,
            'participating': self.participating,
            'preparing_results': self.preparing_results,
            'joined_too_early_date': self.joined_too_early_date,
            'admin_disallowed_chat_id': self.admin_disallowed_chat_id,
            'disallowed_country': self.disallowed_country,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.participating:
            flags |= (1 << 0)
        if self.preparing_results:
            flags |= (1 << 3)
        if self.joined_too_early_date is not None:
            flags |= (1 << 1)
        if self.admin_disallowed_chat_id is not None:
            flags |= (1 << 2)
        if self.disallowed_country is not None:
            flags |= (1 << 4)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.start_date))
        if self.joined_too_early_date is not None:
            buf.write(struct.pack('<i', self.joined_too_early_date))
        if self.admin_disallowed_chat_id is not None:
            buf.write(struct.pack('<q', self.admin_disallowed_chat_id))
        if self.disallowed_country is not None:
            buf.write(TLObject.serialize_bytes(self.disallowed_country))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['participating'] = bool(flags & (1 << 0))
        _args['preparing_results'] = bool(flags & (1 << 3))
        _val_start_date = reader.read_int()
        _args['start_date'] = _val_start_date
        if flags & (1 << 1):
            _val_joined_too_early_date = reader.read_int()
            _args['joined_too_early_date'] = _val_joined_too_early_date
        else:
            _args['joined_too_early_date'] = None
        if flags & (1 << 2):
            _val_admin_disallowed_chat_id = reader.read_long()
            _args['admin_disallowed_chat_id'] = _val_admin_disallowed_chat_id
        else:
            _args['admin_disallowed_chat_id'] = None
        if flags & (1 << 4):
            _val_disallowed_country = reader.tgread_string()
            _args['disallowed_country'] = _val_disallowed_country
        else:
            _args['disallowed_country'] = None
        return cls(**_args)


class GiveawayInfoResults(TLObject):
    """TL type: payments.giveawayInfoResults"""
    CONSTRUCTOR_ID = 0xe175e66f
    SUBCLASS_OF_ID = 0x96a377bd

    def __init__(self, start_date: Optional[datetime], finish_date: Optional[datetime], winners_count: int, winner: Optional[bool] = None, refunded: Optional[bool] = None, gift_code_slug: Optional[str] = None, stars_prize: Optional[int] = None, activated_count: Optional[int] = None):
        self.start_date = start_date
        self.finish_date = finish_date
        self.winners_count = winners_count
        self.winner = winner
        self.refunded = refunded
        self.gift_code_slug = gift_code_slug
        self.stars_prize = stars_prize
        self.activated_count = activated_count

    def to_dict(self):
        return {
            '_': 'GiveawayInfoResults',
            'start_date': self.start_date,
            'finish_date': self.finish_date,
            'winners_count': self.winners_count,
            'winner': self.winner,
            'refunded': self.refunded,
            'gift_code_slug': self.gift_code_slug,
            'stars_prize': self.stars_prize,
            'activated_count': self.activated_count,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.winner:
            flags |= (1 << 0)
        if self.refunded:
            flags |= (1 << 1)
        if self.gift_code_slug is not None:
            flags |= (1 << 3)
        if self.stars_prize is not None:
            flags |= (1 << 4)
        if self.activated_count is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.start_date))
        buf.write(struct.pack('<i', self.finish_date))
        buf.write(struct.pack('<i', self.winners_count))
        if self.gift_code_slug is not None:
            buf.write(TLObject.serialize_bytes(self.gift_code_slug))
        if self.stars_prize is not None:
            buf.write(struct.pack('<q', self.stars_prize))
        if self.activated_count is not None:
            buf.write(struct.pack('<i', self.activated_count))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['winner'] = bool(flags & (1 << 0))
        _args['refunded'] = bool(flags & (1 << 1))
        _val_start_date = reader.read_int()
        _args['start_date'] = _val_start_date
        if flags & (1 << 3):
            _val_gift_code_slug = reader.tgread_string()
            _args['gift_code_slug'] = _val_gift_code_slug
        else:
            _args['gift_code_slug'] = None
        if flags & (1 << 4):
            _val_stars_prize = reader.read_long()
            _args['stars_prize'] = _val_stars_prize
        else:
            _args['stars_prize'] = None
        _val_finish_date = reader.read_int()
        _args['finish_date'] = _val_finish_date
        _val_winners_count = reader.read_int()
        _args['winners_count'] = _val_winners_count
        if flags & (1 << 2):
            _val_activated_count = reader.read_int()
            _args['activated_count'] = _val_activated_count
        else:
            _args['activated_count'] = None
        return cls(**_args)


class PaymentForm(TLObject):
    """TL type: payments.paymentForm"""
    CONSTRUCTOR_ID = 0xa0058751
    SUBCLASS_OF_ID = 0xa0483f19

    def __init__(self, form_id: int, bot_id: int, title: str, description: str, invoice: 'TypeInvoice', provider_id: int, url: str, users: List['TypeUser'], can_save_credentials: Optional[bool] = None, password_missing: Optional[bool] = None, photo: Optional['TypeWebDocument'] = None, native_provider: Optional[str] = None, native_params: Optional['TypeDataJSON'] = None, additional_methods: Optional[List['TypePaymentFormMethod']] = None, saved_info: Optional['TypePaymentRequestedInfo'] = None, saved_credentials: Optional[List['TypePaymentSavedCredentials']] = None):
        self.form_id = form_id
        self.bot_id = bot_id
        self.title = title
        self.description = description
        self.invoice = invoice
        self.provider_id = provider_id
        self.url = url
        self.users = users
        self.can_save_credentials = can_save_credentials
        self.password_missing = password_missing
        self.photo = photo
        self.native_provider = native_provider
        self.native_params = native_params
        self.additional_methods = additional_methods
        self.saved_info = saved_info
        self.saved_credentials = saved_credentials

    def to_dict(self):
        return {
            '_': 'PaymentForm',
            'form_id': self.form_id,
            'bot_id': self.bot_id,
            'title': self.title,
            'description': self.description,
            'invoice': self.invoice,
            'provider_id': self.provider_id,
            'url': self.url,
            'users': self.users,
            'can_save_credentials': self.can_save_credentials,
            'password_missing': self.password_missing,
            'photo': self.photo,
            'native_provider': self.native_provider,
            'native_params': self.native_params,
            'additional_methods': self.additional_methods,
            'saved_info': self.saved_info,
            'saved_credentials': self.saved_credentials,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.can_save_credentials:
            flags |= (1 << 2)
        if self.password_missing:
            flags |= (1 << 3)
        if self.photo is not None:
            flags |= (1 << 5)
        if self.native_params is not None:
            flags |= (1 << 4)
        if self.additional_methods is not None:
            flags |= (1 << 6)
        if self.saved_info is not None:
            flags |= (1 << 0)
        if self.saved_credentials is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<q', self.form_id))
        buf.write(struct.pack('<q', self.bot_id))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.description))
        buf.write(bytes(self.invoice))
        buf.write(struct.pack('<q', self.provider_id))
        buf.write(TLObject.serialize_bytes(self.url))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.photo is not None:
            buf.write(bytes(self.photo))
        if self.native_provider is not None:
            buf.write(TLObject.serialize_bytes(self.native_provider))
        if self.native_params is not None:
            buf.write(bytes(self.native_params))
        if self.additional_methods is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.additional_methods)))
            for item in self.additional_methods:
                buf.write(bytes(self.item))
        if self.saved_info is not None:
            buf.write(bytes(self.saved_info))
        if self.saved_credentials is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.saved_credentials)))
            for item in self.saved_credentials:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['can_save_credentials'] = bool(flags & (1 << 2))
        _args['password_missing'] = bool(flags & (1 << 3))
        _val_form_id = reader.read_long()
        _args['form_id'] = _val_form_id
        _val_bot_id = reader.read_long()
        _args['bot_id'] = _val_bot_id
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_description = reader.tgread_string()
        _args['description'] = _val_description
        if flags & (1 << 5):
            _val_photo = reader.tgread_object()
            _args['photo'] = _val_photo
        else:
            _args['photo'] = None
        _val_invoice = reader.tgread_object()
        _args['invoice'] = _val_invoice
        _val_provider_id = reader.read_long()
        _args['provider_id'] = _val_provider_id
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        if flags & (1 << 4):
            _val_native_provider = reader.tgread_string()
            _args['native_provider'] = _val_native_provider
        else:
            _args['native_provider'] = None
        if flags & (1 << 4):
            _val_native_params = reader.tgread_object()
            _args['native_params'] = _val_native_params
        else:
            _args['native_params'] = None
        if flags & (1 << 6):
            reader.read_int(signed=False)  # skip vector id
            _count_additional_methods = reader.read_int()
            _list_additional_methods = []
            for _ in range(_count_additional_methods):
                _item_additional_methods = reader.tgread_object()
                _list_additional_methods.append(_item_additional_methods)
            _args['additional_methods'] = _list_additional_methods
        else:
            _args['additional_methods'] = None
        if flags & (1 << 0):
            _val_saved_info = reader.tgread_object()
            _args['saved_info'] = _val_saved_info
        else:
            _args['saved_info'] = None
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_saved_credentials = reader.read_int()
            _list_saved_credentials = []
            for _ in range(_count_saved_credentials):
                _item_saved_credentials = reader.tgread_object()
                _list_saved_credentials.append(_item_saved_credentials)
            _args['saved_credentials'] = _list_saved_credentials
        else:
            _args['saved_credentials'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PaymentFormStarGift(TLObject):
    """TL type: payments.paymentFormStarGift"""
    CONSTRUCTOR_ID = 0xb425cfe1
    SUBCLASS_OF_ID = 0xa0483f19

    def __init__(self, form_id: int, invoice: 'TypeInvoice'):
        self.form_id = form_id
        self.invoice = invoice

    def to_dict(self):
        return {
            '_': 'PaymentFormStarGift',
            'form_id': self.form_id,
            'invoice': self.invoice,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.form_id))
        buf.write(bytes(self.invoice))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_form_id = reader.read_long()
        _args['form_id'] = _val_form_id
        _val_invoice = reader.tgread_object()
        _args['invoice'] = _val_invoice
        return cls(**_args)


class PaymentFormStars(TLObject):
    """TL type: payments.paymentFormStars"""
    CONSTRUCTOR_ID = 0x7bf6b15c
    SUBCLASS_OF_ID = 0xa0483f19

    def __init__(self, form_id: int, bot_id: int, title: str, description: str, invoice: 'TypeInvoice', users: List['TypeUser'], photo: Optional['TypeWebDocument'] = None):
        self.form_id = form_id
        self.bot_id = bot_id
        self.title = title
        self.description = description
        self.invoice = invoice
        self.users = users
        self.photo = photo

    def to_dict(self):
        return {
            '_': 'PaymentFormStars',
            'form_id': self.form_id,
            'bot_id': self.bot_id,
            'title': self.title,
            'description': self.description,
            'invoice': self.invoice,
            'users': self.users,
            'photo': self.photo,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.photo is not None:
            flags |= (1 << 5)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<q', self.form_id))
        buf.write(struct.pack('<q', self.bot_id))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.description))
        buf.write(bytes(self.invoice))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.photo is not None:
            buf.write(bytes(self.photo))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_form_id = reader.read_long()
        _args['form_id'] = _val_form_id
        _val_bot_id = reader.read_long()
        _args['bot_id'] = _val_bot_id
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_description = reader.tgread_string()
        _args['description'] = _val_description
        if flags & (1 << 5):
            _val_photo = reader.tgread_object()
            _args['photo'] = _val_photo
        else:
            _args['photo'] = None
        _val_invoice = reader.tgread_object()
        _args['invoice'] = _val_invoice
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PaymentReceipt(TLObject):
    """TL type: payments.paymentReceipt"""
    CONSTRUCTOR_ID = 0x70c4fe03
    SUBCLASS_OF_ID = 0x590093c9

    def __init__(self, date: Optional[datetime], bot_id: int, provider_id: int, title: str, description: str, invoice: 'TypeInvoice', currency: str, total_amount: int, credentials_title: str, users: List['TypeUser'], photo: Optional['TypeWebDocument'] = None, info: Optional['TypePaymentRequestedInfo'] = None, shipping: Optional['TypeShippingOption'] = None, tip_amount: Optional[int] = None):
        self.date = date
        self.bot_id = bot_id
        self.provider_id = provider_id
        self.title = title
        self.description = description
        self.invoice = invoice
        self.currency = currency
        self.total_amount = total_amount
        self.credentials_title = credentials_title
        self.users = users
        self.photo = photo
        self.info = info
        self.shipping = shipping
        self.tip_amount = tip_amount

    def to_dict(self):
        return {
            '_': 'PaymentReceipt',
            'date': self.date,
            'bot_id': self.bot_id,
            'provider_id': self.provider_id,
            'title': self.title,
            'description': self.description,
            'invoice': self.invoice,
            'currency': self.currency,
            'total_amount': self.total_amount,
            'credentials_title': self.credentials_title,
            'users': self.users,
            'photo': self.photo,
            'info': self.info,
            'shipping': self.shipping,
            'tip_amount': self.tip_amount,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.photo is not None:
            flags |= (1 << 2)
        if self.info is not None:
            flags |= (1 << 0)
        if self.shipping is not None:
            flags |= (1 << 1)
        if self.tip_amount is not None:
            flags |= (1 << 3)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.date))
        buf.write(struct.pack('<q', self.bot_id))
        buf.write(struct.pack('<q', self.provider_id))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.description))
        buf.write(bytes(self.invoice))
        buf.write(TLObject.serialize_bytes(self.currency))
        buf.write(struct.pack('<q', self.total_amount))
        buf.write(TLObject.serialize_bytes(self.credentials_title))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.photo is not None:
            buf.write(bytes(self.photo))
        if self.info is not None:
            buf.write(bytes(self.info))
        if self.shipping is not None:
            buf.write(bytes(self.shipping))
        if self.tip_amount is not None:
            buf.write(struct.pack('<q', self.tip_amount))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_date = reader.read_int()
        _args['date'] = _val_date
        _val_bot_id = reader.read_long()
        _args['bot_id'] = _val_bot_id
        _val_provider_id = reader.read_long()
        _args['provider_id'] = _val_provider_id
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_description = reader.tgread_string()
        _args['description'] = _val_description
        if flags & (1 << 2):
            _val_photo = reader.tgread_object()
            _args['photo'] = _val_photo
        else:
            _args['photo'] = None
        _val_invoice = reader.tgread_object()
        _args['invoice'] = _val_invoice
        if flags & (1 << 0):
            _val_info = reader.tgread_object()
            _args['info'] = _val_info
        else:
            _args['info'] = None
        if flags & (1 << 1):
            _val_shipping = reader.tgread_object()
            _args['shipping'] = _val_shipping
        else:
            _args['shipping'] = None
        if flags & (1 << 3):
            _val_tip_amount = reader.read_long()
            _args['tip_amount'] = _val_tip_amount
        else:
            _args['tip_amount'] = None
        _val_currency = reader.tgread_string()
        _args['currency'] = _val_currency
        _val_total_amount = reader.read_long()
        _args['total_amount'] = _val_total_amount
        _val_credentials_title = reader.tgread_string()
        _args['credentials_title'] = _val_credentials_title
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PaymentReceiptStars(TLObject):
    """TL type: payments.paymentReceiptStars"""
    CONSTRUCTOR_ID = 0xdabbf83a
    SUBCLASS_OF_ID = 0x590093c9

    def __init__(self, date: Optional[datetime], bot_id: int, title: str, description: str, invoice: 'TypeInvoice', currency: str, total_amount: int, transaction_id: str, users: List['TypeUser'], photo: Optional['TypeWebDocument'] = None):
        self.date = date
        self.bot_id = bot_id
        self.title = title
        self.description = description
        self.invoice = invoice
        self.currency = currency
        self.total_amount = total_amount
        self.transaction_id = transaction_id
        self.users = users
        self.photo = photo

    def to_dict(self):
        return {
            '_': 'PaymentReceiptStars',
            'date': self.date,
            'bot_id': self.bot_id,
            'title': self.title,
            'description': self.description,
            'invoice': self.invoice,
            'currency': self.currency,
            'total_amount': self.total_amount,
            'transaction_id': self.transaction_id,
            'users': self.users,
            'photo': self.photo,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.photo is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.date))
        buf.write(struct.pack('<q', self.bot_id))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(TLObject.serialize_bytes(self.description))
        buf.write(bytes(self.invoice))
        buf.write(TLObject.serialize_bytes(self.currency))
        buf.write(struct.pack('<q', self.total_amount))
        buf.write(TLObject.serialize_bytes(self.transaction_id))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.photo is not None:
            buf.write(bytes(self.photo))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_date = reader.read_int()
        _args['date'] = _val_date
        _val_bot_id = reader.read_long()
        _args['bot_id'] = _val_bot_id
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        _val_description = reader.tgread_string()
        _args['description'] = _val_description
        if flags & (1 << 2):
            _val_photo = reader.tgread_object()
            _args['photo'] = _val_photo
        else:
            _args['photo'] = None
        _val_invoice = reader.tgread_object()
        _args['invoice'] = _val_invoice
        _val_currency = reader.tgread_string()
        _args['currency'] = _val_currency
        _val_total_amount = reader.read_long()
        _args['total_amount'] = _val_total_amount
        _val_transaction_id = reader.tgread_string()
        _args['transaction_id'] = _val_transaction_id
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PaymentResult(TLObject):
    """TL type: payments.paymentResult"""
    CONSTRUCTOR_ID = 0x4e5f810d
    SUBCLASS_OF_ID = 0x8ae16a9d

    def __init__(self, updates: 'TypeUpdates'):
        self.updates = updates

    def to_dict(self):
        return {
            '_': 'PaymentResult',
            'updates': self.updates,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.updates))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_updates = reader.tgread_object()
        _args['updates'] = _val_updates
        return cls(**_args)


class PaymentVerificationNeeded(TLObject):
    """TL type: payments.paymentVerificationNeeded"""
    CONSTRUCTOR_ID = 0xd8411139
    SUBCLASS_OF_ID = 0x8ae16a9d

    def __init__(self, url: str):
        self.url = url

    def to_dict(self):
        return {
            '_': 'PaymentVerificationNeeded',
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class ResaleStarGifts(TLObject):
    """TL type: payments.resaleStarGifts"""
    CONSTRUCTOR_ID = 0x947a12df
    SUBCLASS_OF_ID = 0xb2dbb7e3

    def __init__(self, count: int, gifts: List['TypeStarGift'], chats: List['TypeChat'], users: List['TypeUser'], next_offset: Optional[str] = None, attributes: Optional[List['TypeStarGiftAttribute']] = None, attributes_hash: Optional[int] = None, counters: Optional[List['TypeStarGiftAttributeCounter']] = None):
        self.count = count
        self.gifts = gifts
        self.chats = chats
        self.users = users
        self.next_offset = next_offset
        self.attributes = attributes
        self.attributes_hash = attributes_hash
        self.counters = counters

    def to_dict(self):
        return {
            '_': 'ResaleStarGifts',
            'count': self.count,
            'gifts': self.gifts,
            'chats': self.chats,
            'users': self.users,
            'next_offset': self.next_offset,
            'attributes': self.attributes,
            'attributes_hash': self.attributes_hash,
            'counters': self.counters,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        if self.attributes_hash is not None:
            flags |= (1 << 1)
        if self.counters is not None:
            flags |= (1 << 2)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.gifts)))
        for item in self.gifts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        if self.attributes is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.attributes)))
            for item in self.attributes:
                buf.write(bytes(self.item))
        if self.attributes_hash is not None:
            buf.write(struct.pack('<q', self.attributes_hash))
        if self.counters is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.counters)))
            for item in self.counters:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_gifts = reader.read_int()
        _list_gifts = []
        for _ in range(_count_gifts):
            _item_gifts = reader.tgread_object()
            _list_gifts.append(_item_gifts)
        _args['gifts'] = _list_gifts
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_attributes = reader.read_int()
            _list_attributes = []
            for _ in range(_count_attributes):
                _item_attributes = reader.tgread_object()
                _list_attributes.append(_item_attributes)
            _args['attributes'] = _list_attributes
        else:
            _args['attributes'] = None
        if flags & (1 << 1):
            _val_attributes_hash = reader.read_long()
            _args['attributes_hash'] = _val_attributes_hash
        else:
            _args['attributes_hash'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        if flags & (1 << 2):
            reader.read_int(signed=False)  # skip vector id
            _count_counters = reader.read_int()
            _list_counters = []
            for _ in range(_count_counters):
                _item_counters = reader.tgread_object()
                _list_counters.append(_item_counters)
            _args['counters'] = _list_counters
        else:
            _args['counters'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SavedInfo(TLObject):
    """TL type: payments.savedInfo"""
    CONSTRUCTOR_ID = 0xfb8fe43c
    SUBCLASS_OF_ID = 0xad3cf146

    def __init__(self, has_saved_credentials: Optional[bool] = None, saved_info: Optional['TypePaymentRequestedInfo'] = None):
        self.has_saved_credentials = has_saved_credentials
        self.saved_info = saved_info

    def to_dict(self):
        return {
            '_': 'SavedInfo',
            'has_saved_credentials': self.has_saved_credentials,
            'saved_info': self.saved_info,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.has_saved_credentials:
            flags |= (1 << 1)
        if self.saved_info is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        if self.saved_info is not None:
            buf.write(bytes(self.saved_info))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['has_saved_credentials'] = bool(flags & (1 << 1))
        if flags & (1 << 0):
            _val_saved_info = reader.tgread_object()
            _args['saved_info'] = _val_saved_info
        else:
            _args['saved_info'] = None
        return cls(**_args)


class SavedStarGifts(TLObject):
    """TL type: payments.savedStarGifts"""
    CONSTRUCTOR_ID = 0x95f389b1
    SUBCLASS_OF_ID = 0xd5112897

    def __init__(self, count: int, gifts: List['TypeSavedStarGift'], chats: List['TypeChat'], users: List['TypeUser'], chat_notifications_enabled: Optional[bool] = None, next_offset: Optional[str] = None):
        self.count = count
        self.gifts = gifts
        self.chats = chats
        self.users = users
        self.chat_notifications_enabled = chat_notifications_enabled
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'SavedStarGifts',
            'count': self.count,
            'gifts': self.gifts,
            'chats': self.chats,
            'users': self.users,
            'chat_notifications_enabled': self.chat_notifications_enabled,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.chat_notifications_enabled is not None:
            flags |= (1 << 1)
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.gifts)))
        for item in self.gifts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.chat_notifications_enabled is not None:
            buf.write(struct.pack('<I', 0x997275b5 if self.chat_notifications_enabled else 0xbc799737))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        if flags & (1 << 1):
            _val_chat_notifications_enabled = reader.tgread_bool()
            _args['chat_notifications_enabled'] = _val_chat_notifications_enabled
        else:
            _args['chat_notifications_enabled'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_gifts = reader.read_int()
        _list_gifts = []
        for _ in range(_count_gifts):
            _item_gifts = reader.tgread_object()
            _list_gifts.append(_item_gifts)
        _args['gifts'] = _list_gifts
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class StarGiftActiveAuctions(TLObject):
    """TL type: payments.starGiftActiveAuctions"""
    CONSTRUCTOR_ID = 0xaef6abbc
    SUBCLASS_OF_ID = 0x917dd0c7

    def __init__(self, auctions: List['TypeStarGiftActiveAuctionState'], users: List['TypeUser'], chats: List['TypeChat']):
        self.auctions = auctions
        self.users = users
        self.chats = chats

    def to_dict(self):
        return {
            '_': 'StarGiftActiveAuctions',
            'auctions': self.auctions,
            'users': self.users,
            'chats': self.chats,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.auctions)))
        for item in self.auctions:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_auctions = reader.read_int()
        _list_auctions = []
        for _ in range(_count_auctions):
            _item_auctions = reader.tgread_object()
            _list_auctions.append(_item_auctions)
        _args['auctions'] = _list_auctions
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        return cls(**_args)


class StarGiftActiveAuctionsNotModified(TLObject):
    """TL type: payments.starGiftActiveAuctionsNotModified"""
    CONSTRUCTOR_ID = 0xdb33dad0
    SUBCLASS_OF_ID = 0x917dd0c7

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'StarGiftActiveAuctionsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class StarGiftAuctionAcquiredGifts(TLObject):
    """TL type: payments.starGiftAuctionAcquiredGifts"""
    CONSTRUCTOR_ID = 0x7d5bd1f0
    SUBCLASS_OF_ID = 0xa7080a1b

    def __init__(self, gifts: List['TypeStarGiftAuctionAcquiredGift'], users: List['TypeUser'], chats: List['TypeChat']):
        self.gifts = gifts
        self.users = users
        self.chats = chats

    def to_dict(self):
        return {
            '_': 'StarGiftAuctionAcquiredGifts',
            'gifts': self.gifts,
            'users': self.users,
            'chats': self.chats,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.gifts)))
        for item in self.gifts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_gifts = reader.read_int()
        _list_gifts = []
        for _ in range(_count_gifts):
            _item_gifts = reader.tgread_object()
            _list_gifts.append(_item_gifts)
        _args['gifts'] = _list_gifts
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        return cls(**_args)


class StarGiftAuctionState(TLObject):
    """TL type: payments.starGiftAuctionState"""
    CONSTRUCTOR_ID = 0x6b39f4ec
    SUBCLASS_OF_ID = 0x1a318599

    def __init__(self, gift: 'TypeStarGift', state: 'TypeStarGiftAuctionState', user_state: 'TypeStarGiftAuctionUserState', timeout: int, users: List['TypeUser'], chats: List['TypeChat']):
        self.gift = gift
        self.state = state
        self.user_state = user_state
        self.timeout = timeout
        self.users = users
        self.chats = chats

    def to_dict(self):
        return {
            '_': 'StarGiftAuctionState',
            'gift': self.gift,
            'state': self.state,
            'user_state': self.user_state,
            'timeout': self.timeout,
            'users': self.users,
            'chats': self.chats,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.gift))
        buf.write(bytes(self.state))
        buf.write(bytes(self.user_state))
        buf.write(struct.pack('<i', self.timeout))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_gift = reader.tgread_object()
        _args['gift'] = _val_gift
        _val_state = reader.tgread_object()
        _args['state'] = _val_state
        _val_user_state = reader.tgread_object()
        _args['user_state'] = _val_user_state
        _val_timeout = reader.read_int()
        _args['timeout'] = _val_timeout
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        return cls(**_args)


class StarGiftCollections(TLObject):
    """TL type: payments.starGiftCollections"""
    CONSTRUCTOR_ID = 0x8a2932f3
    SUBCLASS_OF_ID = 0xf01721ec

    def __init__(self, collections: List['TypeStarGiftCollection']):
        self.collections = collections

    def to_dict(self):
        return {
            '_': 'StarGiftCollections',
            'collections': self.collections,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.collections)))
        for item in self.collections:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_collections = reader.read_int()
        _list_collections = []
        for _ in range(_count_collections):
            _item_collections = reader.tgread_object()
            _list_collections.append(_item_collections)
        _args['collections'] = _list_collections
        return cls(**_args)


class StarGiftCollectionsNotModified(TLObject):
    """TL type: payments.starGiftCollectionsNotModified"""
    CONSTRUCTOR_ID = 0xa0ba4f17
    SUBCLASS_OF_ID = 0xf01721ec

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'StarGiftCollectionsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class StarGiftUpgradeAttributes(TLObject):
    """TL type: payments.starGiftUpgradeAttributes"""
    CONSTRUCTOR_ID = 0x46c6e36f
    SUBCLASS_OF_ID = 0x0f46acef

    def __init__(self, attributes: List['TypeStarGiftAttribute']):
        self.attributes = attributes

    def to_dict(self):
        return {
            '_': 'StarGiftUpgradeAttributes',
            'attributes': self.attributes,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.attributes)))
        for item in self.attributes:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_attributes = reader.read_int()
        _list_attributes = []
        for _ in range(_count_attributes):
            _item_attributes = reader.tgread_object()
            _list_attributes.append(_item_attributes)
        _args['attributes'] = _list_attributes
        return cls(**_args)


class StarGiftUpgradePreview(TLObject):
    """TL type: payments.starGiftUpgradePreview"""
    CONSTRUCTOR_ID = 0x3de1dfed
    SUBCLASS_OF_ID = 0x5e2b68c7

    def __init__(self, sample_attributes: List['TypeStarGiftAttribute'], prices: List['TypeStarGiftUpgradePrice'], next_prices: List['TypeStarGiftUpgradePrice']):
        self.sample_attributes = sample_attributes
        self.prices = prices
        self.next_prices = next_prices

    def to_dict(self):
        return {
            '_': 'StarGiftUpgradePreview',
            'sample_attributes': self.sample_attributes,
            'prices': self.prices,
            'next_prices': self.next_prices,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.sample_attributes)))
        for item in self.sample_attributes:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.prices)))
        for item in self.prices:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.next_prices)))
        for item in self.next_prices:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_sample_attributes = reader.read_int()
        _list_sample_attributes = []
        for _ in range(_count_sample_attributes):
            _item_sample_attributes = reader.tgread_object()
            _list_sample_attributes.append(_item_sample_attributes)
        _args['sample_attributes'] = _list_sample_attributes
        reader.read_int(signed=False)  # skip vector id
        _count_prices = reader.read_int()
        _list_prices = []
        for _ in range(_count_prices):
            _item_prices = reader.tgread_object()
            _list_prices.append(_item_prices)
        _args['prices'] = _list_prices
        reader.read_int(signed=False)  # skip vector id
        _count_next_prices = reader.read_int()
        _list_next_prices = []
        for _ in range(_count_next_prices):
            _item_next_prices = reader.tgread_object()
            _list_next_prices.append(_item_next_prices)
        _args['next_prices'] = _list_next_prices
        return cls(**_args)


class StarGiftWithdrawalUrl(TLObject):
    """TL type: payments.starGiftWithdrawalUrl"""
    CONSTRUCTOR_ID = 0x84aa3a9c
    SUBCLASS_OF_ID = 0xa2822dc5

    def __init__(self, url: str):
        self.url = url

    def to_dict(self):
        return {
            '_': 'StarGiftWithdrawalUrl',
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class StarGifts(TLObject):
    """TL type: payments.starGifts"""
    CONSTRUCTOR_ID = 0x2ed82995
    SUBCLASS_OF_ID = 0x6178d9a4

    def __init__(self, hash: int, gifts: List['TypeStarGift'], chats: List['TypeChat'], users: List['TypeUser']):
        self.hash = hash
        self.gifts = gifts
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'StarGifts',
            'hash': self.hash,
            'gifts': self.gifts,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.hash))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.gifts)))
        for item in self.gifts:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_hash = reader.read_int()
        _args['hash'] = _val_hash
        reader.read_int(signed=False)  # skip vector id
        _count_gifts = reader.read_int()
        _list_gifts = []
        for _ in range(_count_gifts):
            _item_gifts = reader.tgread_object()
            _list_gifts.append(_item_gifts)
        _args['gifts'] = _list_gifts
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class StarGiftsNotModified(TLObject):
    """TL type: payments.starGiftsNotModified"""
    CONSTRUCTOR_ID = 0xa388a368
    SUBCLASS_OF_ID = 0x6178d9a4

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'StarGiftsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class StarsRevenueAdsAccountUrl(TLObject):
    """TL type: payments.starsRevenueAdsAccountUrl"""
    CONSTRUCTOR_ID = 0x394e7f21
    SUBCLASS_OF_ID = 0x4a228b15

    def __init__(self, url: str):
        self.url = url

    def to_dict(self):
        return {
            '_': 'StarsRevenueAdsAccountUrl',
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class StarsRevenueStats(TLObject):
    """TL type: payments.starsRevenueStats"""
    CONSTRUCTOR_ID = 0x6c207376
    SUBCLASS_OF_ID = 0xa54755f3

    def __init__(self, revenue_graph: 'TypeStatsGraph', status: 'TypeStarsRevenueStatus', usd_rate: float, top_hours_graph: Optional['TypeStatsGraph'] = None):
        self.revenue_graph = revenue_graph
        self.status = status
        self.usd_rate = usd_rate
        self.top_hours_graph = top_hours_graph

    def to_dict(self):
        return {
            '_': 'StarsRevenueStats',
            'revenue_graph': self.revenue_graph,
            'status': self.status,
            'usd_rate': self.usd_rate,
            'top_hours_graph': self.top_hours_graph,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.top_hours_graph is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.revenue_graph))
        buf.write(bytes(self.status))
        buf.write(struct.pack('<d', self.usd_rate))
        if self.top_hours_graph is not None:
            buf.write(bytes(self.top_hours_graph))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_top_hours_graph = reader.tgread_object()
            _args['top_hours_graph'] = _val_top_hours_graph
        else:
            _args['top_hours_graph'] = None
        _val_revenue_graph = reader.tgread_object()
        _args['revenue_graph'] = _val_revenue_graph
        _val_status = reader.tgread_object()
        _args['status'] = _val_status
        _val_usd_rate = reader.read_double()
        _args['usd_rate'] = _val_usd_rate
        return cls(**_args)


class StarsRevenueWithdrawalUrl(TLObject):
    """TL type: payments.starsRevenueWithdrawalUrl"""
    CONSTRUCTOR_ID = 0x1dab80b7
    SUBCLASS_OF_ID = 0x8466a0ee

    def __init__(self, url: str):
        self.url = url

    def to_dict(self):
        return {
            '_': 'StarsRevenueWithdrawalUrl',
            'url': self.url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_url = reader.tgread_string()
        _args['url'] = _val_url
        return cls(**_args)


class StarsStatus(TLObject):
    """TL type: payments.starsStatus"""
    CONSTRUCTOR_ID = 0x6c9ce8ed
    SUBCLASS_OF_ID = 0x6e9c1d6f

    def __init__(self, balance: 'TypeStarsAmount', chats: List['TypeChat'], users: List['TypeUser'], subscriptions: Optional[List['TypeStarsSubscription']] = None, subscriptions_next_offset: Optional[str] = None, subscriptions_missing_balance: Optional[int] = None, history: Optional[List['TypeStarsTransaction']] = None, next_offset: Optional[str] = None):
        self.balance = balance
        self.chats = chats
        self.users = users
        self.subscriptions = subscriptions
        self.subscriptions_next_offset = subscriptions_next_offset
        self.subscriptions_missing_balance = subscriptions_missing_balance
        self.history = history
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'StarsStatus',
            'balance': self.balance,
            'chats': self.chats,
            'users': self.users,
            'subscriptions': self.subscriptions,
            'subscriptions_next_offset': self.subscriptions_next_offset,
            'subscriptions_missing_balance': self.subscriptions_missing_balance,
            'history': self.history,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.subscriptions is not None:
            flags |= (1 << 1)
        if self.subscriptions_next_offset is not None:
            flags |= (1 << 2)
        if self.subscriptions_missing_balance is not None:
            flags |= (1 << 4)
        if self.history is not None:
            flags |= (1 << 3)
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.balance))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.subscriptions is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.subscriptions)))
            for item in self.subscriptions:
                buf.write(bytes(self.item))
        if self.subscriptions_next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.subscriptions_next_offset))
        if self.subscriptions_missing_balance is not None:
            buf.write(struct.pack('<q', self.subscriptions_missing_balance))
        if self.history is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.history)))
            for item in self.history:
                buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_balance = reader.tgread_object()
        _args['balance'] = _val_balance
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_subscriptions = reader.read_int()
            _list_subscriptions = []
            for _ in range(_count_subscriptions):
                _item_subscriptions = reader.tgread_object()
                _list_subscriptions.append(_item_subscriptions)
            _args['subscriptions'] = _list_subscriptions
        else:
            _args['subscriptions'] = None
        if flags & (1 << 2):
            _val_subscriptions_next_offset = reader.tgread_string()
            _args['subscriptions_next_offset'] = _val_subscriptions_next_offset
        else:
            _args['subscriptions_next_offset'] = None
        if flags & (1 << 4):
            _val_subscriptions_missing_balance = reader.read_long()
            _args['subscriptions_missing_balance'] = _val_subscriptions_missing_balance
        else:
            _args['subscriptions_missing_balance'] = None
        if flags & (1 << 3):
            reader.read_int(signed=False)  # skip vector id
            _count_history = reader.read_int()
            _list_history = []
            for _ in range(_count_history):
                _item_history = reader.tgread_object()
                _list_history.append(_item_history)
            _args['history'] = _list_history
        else:
            _args['history'] = None
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SuggestedStarRefBots(TLObject):
    """TL type: payments.suggestedStarRefBots"""
    CONSTRUCTOR_ID = 0xb4d5d859
    SUBCLASS_OF_ID = 0x70189243

    def __init__(self, count: int, suggested_bots: List['TypeStarRefProgram'], users: List['TypeUser'], next_offset: Optional[str] = None):
        self.count = count
        self.suggested_bots = suggested_bots
        self.users = users
        self.next_offset = next_offset

    def to_dict(self):
        return {
            '_': 'SuggestedStarRefBots',
            'count': self.count,
            'suggested_bots': self.suggested_bots,
            'users': self.users,
            'next_offset': self.next_offset,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.next_offset is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.suggested_bots)))
        for item in self.suggested_bots:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        if self.next_offset is not None:
            buf.write(TLObject.serialize_bytes(self.next_offset))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_suggested_bots = reader.read_int()
        _list_suggested_bots = []
        for _ in range(_count_suggested_bots):
            _item_suggested_bots = reader.tgread_object()
            _list_suggested_bots.append(_item_suggested_bots)
        _args['suggested_bots'] = _list_suggested_bots
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        if flags & (1 << 0):
            _val_next_offset = reader.tgread_string()
            _args['next_offset'] = _val_next_offset
        else:
            _args['next_offset'] = None
        return cls(**_args)


class UniqueStarGift(TLObject):
    """TL type: payments.uniqueStarGift"""
    CONSTRUCTOR_ID = 0x416c56e8
    SUBCLASS_OF_ID = 0x78b0c5fb

    def __init__(self, gift: 'TypeStarGift', chats: List['TypeChat'], users: List['TypeUser']):
        self.gift = gift
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'UniqueStarGift',
            'gift': self.gift,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.gift))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_gift = reader.tgread_object()
        _args['gift'] = _val_gift
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class UniqueStarGiftValueInfo(TLObject):
    """TL type: payments.uniqueStarGiftValueInfo"""
    CONSTRUCTOR_ID = 0x512fe446
    SUBCLASS_OF_ID = 0x16355bc4

    def __init__(self, currency: str, value: int, initial_sale_date: Optional[datetime], initial_sale_stars: int, initial_sale_price: int, last_sale_on_fragment: Optional[bool] = None, value_is_average: Optional[bool] = None, last_sale_date: Optional[datetime] = None, last_sale_price: Optional[int] = None, floor_price: Optional[int] = None, average_price: Optional[int] = None, listed_count: Optional[int] = None, fragment_listed_count: Optional[int] = None, fragment_listed_url: Optional[str] = None):
        self.currency = currency
        self.value = value
        self.initial_sale_date = initial_sale_date
        self.initial_sale_stars = initial_sale_stars
        self.initial_sale_price = initial_sale_price
        self.last_sale_on_fragment = last_sale_on_fragment
        self.value_is_average = value_is_average
        self.last_sale_date = last_sale_date
        self.last_sale_price = last_sale_price
        self.floor_price = floor_price
        self.average_price = average_price
        self.listed_count = listed_count
        self.fragment_listed_count = fragment_listed_count
        self.fragment_listed_url = fragment_listed_url

    def to_dict(self):
        return {
            '_': 'UniqueStarGiftValueInfo',
            'currency': self.currency,
            'value': self.value,
            'initial_sale_date': self.initial_sale_date,
            'initial_sale_stars': self.initial_sale_stars,
            'initial_sale_price': self.initial_sale_price,
            'last_sale_on_fragment': self.last_sale_on_fragment,
            'value_is_average': self.value_is_average,
            'last_sale_date': self.last_sale_date,
            'last_sale_price': self.last_sale_price,
            'floor_price': self.floor_price,
            'average_price': self.average_price,
            'listed_count': self.listed_count,
            'fragment_listed_count': self.fragment_listed_count,
            'fragment_listed_url': self.fragment_listed_url,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.last_sale_on_fragment:
            flags |= (1 << 1)
        if self.value_is_average:
            flags |= (1 << 6)
        if self.last_sale_price is not None:
            flags |= (1 << 0)
        if self.floor_price is not None:
            flags |= (1 << 2)
        if self.average_price is not None:
            flags |= (1 << 3)
        if self.listed_count is not None:
            flags |= (1 << 4)
        if self.fragment_listed_url is not None:
            flags |= (1 << 5)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.currency))
        buf.write(struct.pack('<q', self.value))
        buf.write(struct.pack('<i', self.initial_sale_date))
        buf.write(struct.pack('<q', self.initial_sale_stars))
        buf.write(struct.pack('<q', self.initial_sale_price))
        if self.last_sale_date is not None:
            buf.write(struct.pack('<i', self.last_sale_date))
        if self.last_sale_price is not None:
            buf.write(struct.pack('<q', self.last_sale_price))
        if self.floor_price is not None:
            buf.write(struct.pack('<q', self.floor_price))
        if self.average_price is not None:
            buf.write(struct.pack('<q', self.average_price))
        if self.listed_count is not None:
            buf.write(struct.pack('<i', self.listed_count))
        if self.fragment_listed_count is not None:
            buf.write(struct.pack('<i', self.fragment_listed_count))
        if self.fragment_listed_url is not None:
            buf.write(TLObject.serialize_bytes(self.fragment_listed_url))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['last_sale_on_fragment'] = bool(flags & (1 << 1))
        _args['value_is_average'] = bool(flags & (1 << 6))
        _val_currency = reader.tgread_string()
        _args['currency'] = _val_currency
        _val_value = reader.read_long()
        _args['value'] = _val_value
        _val_initial_sale_date = reader.read_int()
        _args['initial_sale_date'] = _val_initial_sale_date
        _val_initial_sale_stars = reader.read_long()
        _args['initial_sale_stars'] = _val_initial_sale_stars
        _val_initial_sale_price = reader.read_long()
        _args['initial_sale_price'] = _val_initial_sale_price
        if flags & (1 << 0):
            _val_last_sale_date = reader.read_int()
            _args['last_sale_date'] = _val_last_sale_date
        else:
            _args['last_sale_date'] = None
        if flags & (1 << 0):
            _val_last_sale_price = reader.read_long()
            _args['last_sale_price'] = _val_last_sale_price
        else:
            _args['last_sale_price'] = None
        if flags & (1 << 2):
            _val_floor_price = reader.read_long()
            _args['floor_price'] = _val_floor_price
        else:
            _args['floor_price'] = None
        if flags & (1 << 3):
            _val_average_price = reader.read_long()
            _args['average_price'] = _val_average_price
        else:
            _args['average_price'] = None
        if flags & (1 << 4):
            _val_listed_count = reader.read_int()
            _args['listed_count'] = _val_listed_count
        else:
            _args['listed_count'] = None
        if flags & (1 << 5):
            _val_fragment_listed_count = reader.read_int()
            _args['fragment_listed_count'] = _val_fragment_listed_count
        else:
            _args['fragment_listed_count'] = None
        if flags & (1 << 5):
            _val_fragment_listed_url = reader.tgread_string()
            _args['fragment_listed_url'] = _val_fragment_listed_url
        else:
            _args['fragment_listed_url'] = None
        return cls(**_args)


class ValidatedRequestedInfo(TLObject):
    """TL type: payments.validatedRequestedInfo"""
    CONSTRUCTOR_ID = 0xd1451883
    SUBCLASS_OF_ID = 0x8f8044b7

    def __init__(self, id: Optional[str] = None, shipping_options: Optional[List['TypeShippingOption']] = None):
        self.id = id
        self.shipping_options = shipping_options

    def to_dict(self):
        return {
            '_': 'ValidatedRequestedInfo',
            'id': self.id,
            'shipping_options': self.shipping_options,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.id is not None:
            flags |= (1 << 0)
        if self.shipping_options is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        if self.id is not None:
            buf.write(TLObject.serialize_bytes(self.id))
        if self.shipping_options is not None:
            buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
            buf.write(struct.pack('<i', len(self.shipping_options)))
            for item in self.shipping_options:
                buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        if flags & (1 << 0):
            _val_id = reader.tgread_string()
            _args['id'] = _val_id
        else:
            _args['id'] = None
        if flags & (1 << 1):
            reader.read_int(signed=False)  # skip vector id
            _count_shipping_options = reader.read_int()
            _list_shipping_options = []
            for _ in range(_count_shipping_options):
                _item_shipping_options = reader.tgread_object()
                _list_shipping_options.append(_item_shipping_options)
            _args['shipping_options'] = _list_shipping_options
        else:
            _args['shipping_options'] = None
        return cls(**_args)




"""Various helpers for tglib."""
import os
import struct
import logging
import asyncio
from hashlib import sha1

_log = logging.getLogger(__name__)


def generate_random_long(signed: bool = True) -> int:
    """Generate a random 64-bit integer."""
    return int.from_bytes(os.urandom(8), signed=signed, byteorder='little')


def generate_key_data_from_nonce(server_nonce: int, new_nonce: int):
    """
    Generate AES key and IV from nonces for the DH exchange.
    See https://core.telegram.org/mtproto/auth_key#server-dh-parameters
    """
    server_nonce_bytes = server_nonce.to_bytes(16, 'little', signed=True)
    new_nonce_bytes = new_nonce.to_bytes(32, 'little', signed=True)

    hash1 = sha1(new_nonce_bytes + server_nonce_bytes).digest()
    hash2 = sha1(server_nonce_bytes + new_nonce_bytes).digest()
    hash3 = sha1(new_nonce_bytes + new_nonce_bytes).digest()

    key = hash1 + hash2[:12]
    iv = hash2[12:] + hash3 + new_nonce_bytes[:4]
    return key, iv


async def retry_range(retries: int, *, delay: float = 1.0):
    """Async generator that yields attempt numbers, sleeping between them."""
    for attempt in range(retries):
        yield attempt
        if attempt < retries - 1:
            await asyncio.sleep(delay)


def add_surrogate(text: str) -> str:
    """Add surrogates for SMP characters (Telegram uses UTF-16 offsets)."""
    return ''.join(
        ''.join(chr(y) for y in struct.unpack('<HH', x.encode('utf-16le')))
        if (0x10000 <= ord(x) <= 0x10FFFF) else x
        for x in text
    )


def del_surrogate(text: str) -> str:
    return text.encode('utf-16', 'surrogatepass').decode('utf-16')


def get_peer_id(peer) -> int:
    """Extract the numeric peer ID from various peer types."""
    from .tl import types
    if isinstance(peer, types.PeerUser):
        return peer.user_id
    if isinstance(peer, types.PeerChat):
        return -peer.chat_id
    if isinstance(peer, types.PeerChannel):
        return int(f'-100{peer.channel_id}')
    return 0


def is_list_like(obj) -> bool:
    """Check if an object is list-like (but not str/bytes)."""
    return (
        hasattr(obj, '__iter__')
        and not isinstance(obj, (str, bytes))
    )



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class AdminLogResults(TLObject):
    """TL type: channels.adminLogResults"""
    CONSTRUCTOR_ID = 0xed8af74d
    SUBCLASS_OF_ID = 0x51f076bc

    def __init__(self, events: List['TypeChannelAdminLogEvent'], chats: List['TypeChat'], users: List['TypeUser']):
        self.events = events
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'AdminLogResults',
            'events': self.events,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.events)))
        for item in self.events:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_events = reader.read_int()
        _list_events = []
        for _ in range(_count_events):
            _item_events = reader.tgread_object()
            _list_events.append(_item_events)
        _args['events'] = _list_events
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ChannelParticipant(TLObject):
    """TL type: channels.channelParticipant"""
    CONSTRUCTOR_ID = 0xdfb80317
    SUBCLASS_OF_ID = 0x6658151a

    def __init__(self, participant: 'TypeChannelParticipant', chats: List['TypeChat'], users: List['TypeUser']):
        self.participant = participant
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'ChannelParticipant',
            'participant': self.participant,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.participant))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_participant = reader.tgread_object()
        _args['participant'] = _val_participant
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ChannelParticipants(TLObject):
    """TL type: channels.channelParticipants"""
    CONSTRUCTOR_ID = 0x9ab0feaf
    SUBCLASS_OF_ID = 0xe60a6e64

    def __init__(self, count: int, participants: List['TypeChannelParticipant'], chats: List['TypeChat'], users: List['TypeUser']):
        self.count = count
        self.participants = participants
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'ChannelParticipants',
            'count': self.count,
            'participants': self.participants,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.participants)))
        for item in self.participants:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_participants = reader.read_int()
        _list_participants = []
        for _ in range(_count_participants):
            _item_participants = reader.tgread_object()
            _list_participants.append(_item_participants)
        _args['participants'] = _list_participants
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class ChannelParticipantsNotModified(TLObject):
    """TL type: channels.channelParticipantsNotModified"""
    CONSTRUCTOR_ID = 0xf0173fe9
    SUBCLASS_OF_ID = 0xe60a6e64

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'ChannelParticipantsNotModified',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class SendAsPeers(TLObject):
    """TL type: channels.sendAsPeers"""
    CONSTRUCTOR_ID = 0xf496b0c6
    SUBCLASS_OF_ID = 0x38cb8d21

    def __init__(self, peers: List['TypeSendAsPeer'], chats: List['TypeChat'], users: List['TypeUser']):
        self.peers = peers
        self.chats = chats
        self.users = users

    def to_dict(self):
        return {
            '_': 'SendAsPeers',
            'peers': self.peers,
            'chats': self.chats,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.peers)))
        for item in self.peers:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.chats)))
        for item in self.chats:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_peers = reader.read_int()
        _list_peers = []
        for _ in range(_count_peers):
            _item_peers = reader.tgread_object()
            _list_peers.append(_item_peers)
        _args['peers'] = _list_peers
        reader.read_int(signed=False)  # skip vector id
        _count_chats = reader.read_int()
        _list_chats = []
        for _ in range(_count_chats):
            _item_chats = reader.tgread_object()
            _list_chats.append(_item_chats)
        _args['chats'] = _list_chats
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class SponsoredMessageReportResultAdsHidden(TLObject):
    """TL type: channels.sponsoredMessageReportResultAdsHidden"""
    CONSTRUCTOR_ID = 0x3e3bcf2f
    SUBCLASS_OF_ID = 0x26231822

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SponsoredMessageReportResultAdsHidden',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class SponsoredMessageReportResultChooseOption(TLObject):
    """TL type: channels.sponsoredMessageReportResultChooseOption"""
    CONSTRUCTOR_ID = 0x846f9e42
    SUBCLASS_OF_ID = 0x26231822

    def __init__(self, title: str, options: List['TypeSponsoredMessageReportOption']):
        self.title = title
        self.options = options

    def to_dict(self):
        return {
            '_': 'SponsoredMessageReportResultChooseOption',
            'title': self.title,
            'options': self.options,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.title))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.options)))
        for item in self.options:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_title = reader.tgread_string()
        _args['title'] = _val_title
        reader.read_int(signed=False)  # skip vector id
        _count_options = reader.read_int()
        _list_options = []
        for _ in range(_count_options):
            _item_options = reader.tgread_object()
            _list_options.append(_item_options)
        _args['options'] = _list_options
        return cls(**_args)


class SponsoredMessageReportResultReported(TLObject):
    """TL type: channels.sponsoredMessageReportResultReported"""
    CONSTRUCTOR_ID = 0xad798849
    SUBCLASS_OF_ID = 0x26231822

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'SponsoredMessageReportResultReported',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class SuggestedShortName(TLObject):
    """TL type: stickers.suggestedShortName"""
    CONSTRUCTOR_ID = 0x85fea03f
    SUBCLASS_OF_ID = 0xc44a4b21

    def __init__(self, short_name: str):
        self.short_name = short_name

    def to_dict(self):
        return {
            '_': 'SuggestedShortName',
            'short_name': self.short_name,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.short_name))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_short_name = reader.tgread_string()
        _args['short_name'] = _val_short_name
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class Photo(TLObject):
    """TL type: photos.photo"""
    CONSTRUCTOR_ID = 0x20212ca8
    SUBCLASS_OF_ID = 0xc292bd24

    def __init__(self, photo: 'TypePhoto', users: List['TypeUser']):
        self.photo = photo
        self.users = users

    def to_dict(self):
        return {
            '_': 'Photo',
            'photo': self.photo,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.photo))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_photo = reader.tgread_object()
        _args['photo'] = _val_photo
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class Photos(TLObject):
    """TL type: photos.photos"""
    CONSTRUCTOR_ID = 0x8dca6aa5
    SUBCLASS_OF_ID = 0x27cfb967

    def __init__(self, photos: List['TypePhoto'], users: List['TypeUser']):
        self.photos = photos
        self.users = users

    def to_dict(self):
        return {
            '_': 'Photos',
            'photos': self.photos,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.photos)))
        for item in self.photos:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_photos = reader.read_int()
        _list_photos = []
        for _ in range(_count_photos):
            _item_photos = reader.tgread_object()
            _list_photos.append(_item_photos)
        _args['photos'] = _list_photos
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)


class PhotosSlice(TLObject):
    """TL type: photos.photosSlice"""
    CONSTRUCTOR_ID = 0x15051f54
    SUBCLASS_OF_ID = 0x27cfb967

    def __init__(self, count: int, photos: List['TypePhoto'], users: List['TypeUser']):
        self.count = count
        self.photos = photos
        self.users = users

    def to_dict(self):
        return {
            '_': 'PhotosSlice',
            'count': self.count,
            'photos': self.photos,
            'users': self.users,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.count))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.photos)))
        for item in self.photos:
            buf.write(bytes(self.item))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.users)))
        for item in self.users:
            buf.write(bytes(self.item))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_count = reader.read_int()
        _args['count'] = _val_count
        reader.read_int(signed=False)  # skip vector id
        _count_photos = reader.read_int()
        _list_photos = []
        for _ in range(_count_photos):
            _item_photos = reader.tgread_object()
            _list_photos.append(_item_photos)
        _args['photos'] = _list_photos
        reader.read_int(signed=False)  # skip vector id
        _count_users = reader.read_int()
        _list_users = []
        for _ in range(_count_users):
            _item_users = reader.tgread_object()
            _list_users.append(_item_users)
        _args['users'] = _list_users
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class FinishJobRequest(TLRequest):
    """TL type: smsjobs.finishJob"""
    CONSTRUCTOR_ID = 0x4f1ebf24
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, job_id: str, error: Optional[str] = None):
        self.job_id = job_id
        self.error = error

    def to_dict(self):
        return {
            '_': 'FinishJobRequest',
            'job_id': self.job_id,
            'error': self.error,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.error is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.job_id))
        if self.error is not None:
            buf.write(TLObject.serialize_bytes(self.error))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_job_id = reader.tgread_string()
        _args['job_id'] = _val_job_id
        if flags & (1 << 0):
            _val_error = reader.tgread_string()
            _args['error'] = _val_error
        else:
            _args['error'] = None
        return cls(**_args)


class GetSmsJobRequest(TLRequest):
    """TL type: smsjobs.getSmsJob"""
    CONSTRUCTOR_ID = 0x778d902f
    SUBCLASS_OF_ID = 0x1f24187e

    def __init__(self, job_id: str):
        self.job_id = job_id

    def to_dict(self):
        return {
            '_': 'GetSmsJobRequest',
            'job_id': self.job_id,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.job_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_job_id = reader.tgread_string()
        _args['job_id'] = _val_job_id
        return cls(**_args)


class GetStatusRequest(TLRequest):
    """TL type: smsjobs.getStatus"""
    CONSTRUCTOR_ID = 0x10a698e8
    SUBCLASS_OF_ID = 0xcd8f2b25

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'GetStatusRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class IsEligibleToJoinRequest(TLRequest):
    """TL type: smsjobs.isEligibleToJoin"""
    CONSTRUCTOR_ID = 0x0edc39d0
    SUBCLASS_OF_ID = 0x5eb760a6

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'IsEligibleToJoinRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class JoinRequest(TLRequest):
    """TL type: smsjobs.join"""
    CONSTRUCTOR_ID = 0xa74ece2d
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'JoinRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class LeaveRequest(TLRequest):
    """TL type: smsjobs.leave"""
    CONSTRUCTOR_ID = 0x9898ad73
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self):
        pass

    def to_dict(self):
        return {
            '_': 'LeaveRequest',
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        return cls()


class UpdateSettingsRequest(TLRequest):
    """TL type: smsjobs.updateSettings"""
    CONSTRUCTOR_ID = 0x093fa0bf
    SUBCLASS_OF_ID = 0xf5b399ac

    def __init__(self, allow_international: Optional[bool] = None):
        self.allow_international = allow_international

    def to_dict(self):
        return {
            '_': 'UpdateSettingsRequest',
            'allow_international': self.allow_international,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.allow_international:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['allow_international'] = bool(flags & (1 << 0))
        return cls(**_args)




"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class InitConnectionRequest(TLRequest):
    """TL type: initConnection"""
    CONSTRUCTOR_ID = 0xc1cd5ea9
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, api_id: int, device_model: str, system_version: str, app_version: str, system_lang_code: str, lang_pack: str, lang_code: str, query: 'TypeX', proxy: Optional['TypeInputClientProxy'] = None, params: Optional['TypeJSONValue'] = None):
        self.api_id = api_id
        self.device_model = device_model
        self.system_version = system_version
        self.app_version = app_version
        self.system_lang_code = system_lang_code
        self.lang_pack = lang_pack
        self.lang_code = lang_code
        self.query = query
        self.proxy = proxy
        self.params = params

    def to_dict(self):
        return {
            '_': 'InitConnectionRequest',
            'api_id': self.api_id,
            'device_model': self.device_model,
            'system_version': self.system_version,
            'app_version': self.app_version,
            'system_lang_code': self.system_lang_code,
            'lang_pack': self.lang_pack,
            'lang_code': self.lang_code,
            'query': self.query,
            'proxy': self.proxy,
            'params': self.params,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.proxy is not None:
            flags |= (1 << 0)
        if self.params is not None:
            flags |= (1 << 1)
        buf.write(struct.pack('<I', flags))
        buf.write(struct.pack('<i', self.api_id))
        buf.write(TLObject.serialize_bytes(self.device_model))
        buf.write(TLObject.serialize_bytes(self.system_version))
        buf.write(TLObject.serialize_bytes(self.app_version))
        buf.write(TLObject.serialize_bytes(self.system_lang_code))
        buf.write(TLObject.serialize_bytes(self.lang_pack))
        buf.write(TLObject.serialize_bytes(self.lang_code))
        buf.write(bytes(self.query))
        if self.proxy is not None:
            buf.write(bytes(self.proxy))
        if self.params is not None:
            buf.write(bytes(self.params))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_api_id = reader.read_int()
        _args['api_id'] = _val_api_id
        _val_device_model = reader.tgread_string()
        _args['device_model'] = _val_device_model
        _val_system_version = reader.tgread_string()
        _args['system_version'] = _val_system_version
        _val_app_version = reader.tgread_string()
        _args['app_version'] = _val_app_version
        _val_system_lang_code = reader.tgread_string()
        _args['system_lang_code'] = _val_system_lang_code
        _val_lang_pack = reader.tgread_string()
        _args['lang_pack'] = _val_lang_pack
        _val_lang_code = reader.tgread_string()
        _args['lang_code'] = _val_lang_code
        if flags & (1 << 0):
            _val_proxy = reader.tgread_object()
            _args['proxy'] = _val_proxy
        else:
            _args['proxy'] = None
        if flags & (1 << 1):
            _val_params = reader.tgread_object()
            _args['params'] = _val_params
        else:
            _args['params'] = None
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeAfterMsgRequest(TLRequest):
    """TL type: invokeAfterMsg"""
    CONSTRUCTOR_ID = 0xcb9f372d
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, msg_id: int, query: 'TypeX'):
        self.msg_id = msg_id
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeAfterMsgRequest',
            'msg_id': self.msg_id,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.msg_id))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_msg_id = reader.read_long()
        _args['msg_id'] = _val_msg_id
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeAfterMsgsRequest(TLRequest):
    """TL type: invokeAfterMsgs"""
    CONSTRUCTOR_ID = 0x3dc4b4f0
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, msg_ids: List[int], query: 'TypeX'):
        self.msg_ids = msg_ids
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeAfterMsgsRequest',
            'msg_ids': self.msg_ids,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', 0x1cb5c415))  # vector id
        buf.write(struct.pack('<i', len(self.msg_ids)))
        for item in self.msg_ids:
            buf.write(struct.pack('<q', self.item))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        reader.read_int(signed=False)  # skip vector id
        _count_msg_ids = reader.read_int()
        _list_msg_ids = []
        for _ in range(_count_msg_ids):
            _item_msg_ids = reader.read_long()
            _list_msg_ids.append(_item_msg_ids)
        _args['msg_ids'] = _list_msg_ids
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithApnsSecretRequest(TLRequest):
    """TL type: invokeWithApnsSecret"""
    CONSTRUCTOR_ID = 0x0dae54f8
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, nonce: str, secret: str, query: 'TypeX'):
        self.nonce = nonce
        self.secret = secret
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithApnsSecretRequest',
            'nonce': self.nonce,
            'secret': self.secret,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.nonce))
        buf.write(TLObject.serialize_bytes(self.secret))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_nonce = reader.tgread_string()
        _args['nonce'] = _val_nonce
        _val_secret = reader.tgread_string()
        _args['secret'] = _val_secret
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithBusinessConnectionRequest(TLRequest):
    """TL type: invokeWithBusinessConnection"""
    CONSTRUCTOR_ID = 0xdd289f8e
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, connection_id: str, query: 'TypeX'):
        self.connection_id = connection_id
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithBusinessConnectionRequest',
            'connection_id': self.connection_id,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.connection_id))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_connection_id = reader.tgread_string()
        _args['connection_id'] = _val_connection_id
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithGooglePlayIntegrityRequest(TLRequest):
    """TL type: invokeWithGooglePlayIntegrity"""
    CONSTRUCTOR_ID = 0x1df92984
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, nonce: str, token: str, query: 'TypeX'):
        self.nonce = nonce
        self.token = token
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithGooglePlayIntegrityRequest',
            'nonce': self.nonce,
            'token': self.token,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.nonce))
        buf.write(TLObject.serialize_bytes(self.token))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_nonce = reader.tgread_string()
        _args['nonce'] = _val_nonce
        _val_token = reader.tgread_string()
        _args['token'] = _val_token
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithLayerRequest(TLRequest):
    """TL type: invokeWithLayer"""
    CONSTRUCTOR_ID = 0xda9b0d0d
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, layer: int, query: 'TypeX'):
        self.layer = layer
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithLayerRequest',
            'layer': self.layer,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<i', self.layer))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_layer = reader.read_int()
        _args['layer'] = _val_layer
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithMessagesRangeRequest(TLRequest):
    """TL type: invokeWithMessagesRange"""
    CONSTRUCTOR_ID = 0x365275f2
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, range: 'TypeMessageRange', query: 'TypeX'):
        self.range = range
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithMessagesRangeRequest',
            'range': self.range,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.range))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_range = reader.tgread_object()
        _args['range'] = _val_range
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithReCaptchaRequest(TLRequest):
    """TL type: invokeWithReCaptcha"""
    CONSTRUCTOR_ID = 0xadbb0f94
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, token: str, query: 'TypeX'):
        self.token = token
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithReCaptchaRequest',
            'token': self.token,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(TLObject.serialize_bytes(self.token))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_token = reader.tgread_string()
        _args['token'] = _val_token
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithTakeoutRequest(TLRequest):
    """TL type: invokeWithTakeout"""
    CONSTRUCTOR_ID = 0xaca9fd2e
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, takeout_id: int, query: 'TypeX'):
        self.takeout_id = takeout_id
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithTakeoutRequest',
            'takeout_id': self.takeout_id,
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(struct.pack('<q', self.takeout_id))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_takeout_id = reader.read_long()
        _args['takeout_id'] = _val_takeout_id
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)


class InvokeWithoutUpdatesRequest(TLRequest):
    """TL type: invokeWithoutUpdates"""
    CONSTRUCTOR_ID = 0xbf9459b7
    SUBCLASS_OF_ID = 0xb7b2364b

    def __init__(self, query: 'TypeX'):
        self.query = query

    def to_dict(self):
        return {
            '_': 'InvokeWithoutUpdatesRequest',
            'query': self.query,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.query))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_query = reader.tgread_object()
        _args['query'] = _val_query
        return cls(**_args)




"""TL core protocol types."""
import gzip
import struct

from ..tlobject import TLObject, TLRequest


class GzipPacked(TLObject):
    CONSTRUCTOR_ID = 0x3072cfa1

    def __init__(self, data: bytes):
        self.data = data

    @staticmethod
    def gzip_if_smaller(content_related: bool, data: bytes) -> bytes:
        """Return gzip-packed data if it's smaller, otherwise original."""
        if content_related:
            try:
                gzipped = gzip.compress(data)
                if len(gzipped) < len(data):
                    packed = GzipPacked(gzipped)
                    return bytes(packed)
            except Exception:
                pass
        return data

    def _bytes(self) -> bytes:
        return (struct.pack('<I', self.CONSTRUCTOR_ID) +
                TLObject.serialize_bytes(self.data))

    @classmethod
    def from_reader(cls, reader):
        data = reader.tgread_bytes()
        return cls(gzip.decompress(data))

    def to_dict(self):
        return {'_': 'GzipPacked', 'data': self.data}


class MessageContainer(TLObject):
    CONSTRUCTOR_ID = 0x73f1f8dc
    MAXIMUM_SIZE = 1044456  # 1MB
    MAXIMUM_LENGTH = 100

    def __init__(self, messages):
        self.messages = messages

    @classmethod
    def from_reader(cls, reader):
        count = reader.read_int()
        messages = []
        for _ in range(count):
            from .tlmessage import TLMessage
            msg_id = reader.read_long()
            seq_no = reader.read_int()
            length = reader.read_int()
            obj = reader.tgread_object()
            messages.append(TLMessage(msg_id, seq_no, obj))
        return cls(messages)

    def to_dict(self):
        return {'_': 'MessageContainer', 'messages': self.messages}


class RpcResult(TLObject):
    CONSTRUCTOR_ID = 0xf35c6d01

    def __init__(self, req_msg_id: int, body):
        self.req_msg_id = req_msg_id
        self.body = body
        self.error = None

    @classmethod
    def from_reader(cls, reader):
        req_msg_id = reader.read_long()
        inner_code = reader.read_int(signed=False)

        if inner_code == 0x2144ca19:  # rpc_error
            error_code = reader.read_int()
            error_message = reader.tgread_string()
            result = cls(req_msg_id, None)
            result.error = (error_code, error_message)
        elif inner_code == GzipPacked.CONSTRUCTOR_ID:
            packed_data = reader.tgread_bytes()
            unpacked = gzip.decompress(packed_data)
            from ...extensions import BinaryReader
            with BinaryReader(unpacked) as inner:
                result = cls(req_msg_id, inner.tgread_object())
        else:
            reader.seek(-4)
            result = cls(req_msg_id, reader.tgread_object())

        return result

    def to_dict(self):
        return {'_': 'RpcResult', 'req_msg_id': self.req_msg_id, 'body': self.body}



"""RequestState - tracks the lifecycle of a single pending request."""
import asyncio


class RequestState:
    """Wraps a TLRequest with its future result and tracking info."""

    def __init__(self, request, *, after=None):
        self.request = request
        self.future = asyncio.get_event_loop().create_future()
        self.msg_id = None
        self.after = after  # Optional[RequestState] for ordered sending

    def __repr__(self):
        return f'RequestState({type(self.request).__name__}, msg_id={self.msg_id})'



"""File auto-generated by tglib TL generator. Do not edit manually."""
from ..tlobject import TLObject
from ..tlobject import TLRequest
from typing import Optional, List, Union
import os
import struct
from datetime import datetime


class GetBroadcastStatsRequest(TLRequest):
    """TL type: stats.getBroadcastStats"""
    CONSTRUCTOR_ID = 0xab42441a
    SUBCLASS_OF_ID = 0x7ff25428

    def __init__(self, channel: 'TypeInputChannel', dark: Optional[bool] = None):
        self.channel = channel
        self.dark = dark

    def to_dict(self):
        return {
            '_': 'GetBroadcastStatsRequest',
            'channel': self.channel,
            'dark': self.dark,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.dark:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['dark'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class GetMegagroupStatsRequest(TLRequest):
    """TL type: stats.getMegagroupStats"""
    CONSTRUCTOR_ID = 0xdcdf8607
    SUBCLASS_OF_ID = 0x5b59be8d

    def __init__(self, channel: 'TypeInputChannel', dark: Optional[bool] = None):
        self.channel = channel
        self.dark = dark

    def to_dict(self):
        return {
            '_': 'GetMegagroupStatsRequest',
            'channel': self.channel,
            'dark': self.dark,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.dark:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['dark'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        return cls(**_args)


class GetMessagePublicForwardsRequest(TLRequest):
    """TL type: stats.getMessagePublicForwards"""
    CONSTRUCTOR_ID = 0x5f150144
    SUBCLASS_OF_ID = 0xa7283211

    def __init__(self, channel: 'TypeInputChannel', msg_id: int, offset: str, limit: int):
        self.channel = channel
        self.msg_id = msg_id
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetMessagePublicForwardsRequest',
            'channel': self.channel,
            'msg_id': self.msg_id,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.msg_id))
        buf.write(TLObject.serialize_bytes(self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_msg_id = reader.read_int()
        _args['msg_id'] = _val_msg_id
        _val_offset = reader.tgread_string()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetMessageStatsRequest(TLRequest):
    """TL type: stats.getMessageStats"""
    CONSTRUCTOR_ID = 0xb6e0a3f5
    SUBCLASS_OF_ID = 0x9604a322

    def __init__(self, channel: 'TypeInputChannel', msg_id: int, dark: Optional[bool] = None):
        self.channel = channel
        self.msg_id = msg_id
        self.dark = dark

    def to_dict(self):
        return {
            '_': 'GetMessageStatsRequest',
            'channel': self.channel,
            'msg_id': self.msg_id,
            'dark': self.dark,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.dark:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.channel))
        buf.write(struct.pack('<i', self.msg_id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['dark'] = bool(flags & (1 << 0))
        _val_channel = reader.tgread_object()
        _args['channel'] = _val_channel
        _val_msg_id = reader.read_int()
        _args['msg_id'] = _val_msg_id
        return cls(**_args)


class GetStoryPublicForwardsRequest(TLRequest):
    """TL type: stats.getStoryPublicForwards"""
    CONSTRUCTOR_ID = 0xa6437ef6
    SUBCLASS_OF_ID = 0xa7283211

    def __init__(self, peer: 'TypeInputPeer', id: int, offset: str, limit: int):
        self.peer = peer
        self.id = id
        self.offset = offset
        self.limit = limit

    def to_dict(self):
        return {
            '_': 'GetStoryPublicForwardsRequest',
            'peer': self.peer,
            'id': self.id,
            'offset': self.offset,
            'limit': self.limit,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', self.id))
        buf.write(TLObject.serialize_bytes(self.offset))
        buf.write(struct.pack('<i', self.limit))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_id = reader.read_int()
        _args['id'] = _val_id
        _val_offset = reader.tgread_string()
        _args['offset'] = _val_offset
        _val_limit = reader.read_int()
        _args['limit'] = _val_limit
        return cls(**_args)


class GetStoryStatsRequest(TLRequest):
    """TL type: stats.getStoryStats"""
    CONSTRUCTOR_ID = 0x374fef40
    SUBCLASS_OF_ID = 0x8b4d43d4

    def __init__(self, peer: 'TypeInputPeer', id: int, dark: Optional[bool] = None):
        self.peer = peer
        self.id = id
        self.dark = dark

    def to_dict(self):
        return {
            '_': 'GetStoryStatsRequest',
            'peer': self.peer,
            'id': self.id,
            'dark': self.dark,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.dark:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(bytes(self.peer))
        buf.write(struct.pack('<i', self.id))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _args['dark'] = bool(flags & (1 << 0))
        _val_peer = reader.tgread_object()
        _args['peer'] = _val_peer
        _val_id = reader.read_int()
        _args['id'] = _val_id
        return cls(**_args)


class LoadAsyncGraphRequest(TLRequest):
    """TL type: stats.loadAsyncGraph"""
    CONSTRUCTOR_ID = 0x621d5fa0
    SUBCLASS_OF_ID = 0x9b903153

    def __init__(self, token: str, x: Optional[int] = None):
        self.token = token
        self.x = x

    def to_dict(self):
        return {
            '_': 'LoadAsyncGraphRequest',
            'token': self.token,
            'x': self.x,
        }

    def _bytes(self) -> bytes:
        import io
        buf = io.BytesIO()
        buf.write(struct.pack('<I', self.CONSTRUCTOR_ID))
        flags = 0
        if self.x is not None:
            flags |= (1 << 0)
        buf.write(struct.pack('<I', flags))
        buf.write(TLObject.serialize_bytes(self.token))
        if self.x is not None:
            buf.write(struct.pack('<q', self.x))
        return buf.getvalue()

    @classmethod
    def from_reader(cls, reader):
        _args = {}
        flags = reader.read_int(signed=False)
        _val_token = reader.tgread_string()
        _args['token'] = _val_token
        if flags & (1 << 0):
            _val_x = reader.read_long()
            _args['x'] = _val_x
        else:
            _args['x'] = None
        return cls(**_args)

